@@ -5,6 +5,7 @@
 """
 The core of grab package: the Grab class.
 """
+from __future__ import absolute_import
 import logging
 import os
 from random import randint
@@ -12,26 +13,19 @@ from copy import copy, deepcopy
 import threading
 import itertools
 import collections
-try:
-    from urlparse import urljoin
-except ImportError:
-    from urllib.parse import urljoin
+from six.moves.urllib.parse import urljoin
 import email
 from datetime import datetime
 import weakref
+import six
 
-from grab.tools.html import find_refresh_url, find_base_url
+from weblib.html import find_refresh_url, find_base_url
 from grab.document import Document
 from grab import error
-from grab.tools.http import normalize_http_values
+from weblib.http import normalize_http_values
 from grab.cookie import CookieManager
-from grab.proxy import ProxyList, parse_proxy_line
+from grab.proxylist import ProxyList, parse_proxy_line
 from grab.deprecated import DeprecatedThings
-from grab.kit_interface import GrabKitInterface
-from grab.ext.form import FormExtension
-
-from grab.util.py2old_support import *
-from grab.util.py3k_support import *
 
 __all__ = ('Grab',)
 # This counter will used in enumerating network queries.
@@ -46,7 +40,6 @@ REQUEST_COUNTER = itertools.count(1)
 
 GLOBAL_STATE = {
     'dom_build_time': 0,
-    'selector_time': 0,
 }
 MUTABLE_CONFIG_KEYS = ['post', 'multipart_post', 'headers', 'cookies']
 TRANSPORT_CACHE = {}
@@ -197,7 +190,7 @@ class Grab(DeprecatedThings):
 
                  # Dirty hack to make it possible to inherit Grab from
                  # multiple base classes with __slots__
-                 '_doc', '_kit',
+                 '_doc',
                  )
 
     # Attributes which should be processed when clone
@@ -245,7 +238,7 @@ class Grab(DeprecatedThings):
 
     def setup_transport(self, transport_param):
         self.transport_param = transport_param
-        if isinstance(transport_param, basestring):
+        if isinstance(transport_param, six.string_types):
             mod_path, cls_name = transport_param.rsplit('.', 1)
             try:
                 cls = TRANSPORT_CACHE[(mod_path, cls_name)]
@@ -276,9 +269,6 @@ class Grab(DeprecatedThings):
         self.request_method = None
         self.transport.reset()
 
-        # KIT
-        self._kit = None
-
     def clone(self, **kwargs):
         """
         Create clone of Grab instance.
@@ -347,13 +337,13 @@ class Grab(DeprecatedThings):
         """
 
         if 'hammer_mode' in kwargs:
-            logging.error('Option hammer_mode is deprecated. Grab does not '
-                          'support hammer mode anymore.')
+            logger.error('Option hammer_mode is deprecated. Grab does not '
+                         'support hammer mode anymore.')
             del kwargs['hammer_mode']
 
         if 'hammer_timeouts' in kwargs:
-            logging.error('Option hammer_timeouts is deprecated. Grab does not'
-                          ' support hammer mode anymore.')
+            logger.error('Option hammer_timeouts is deprecated. Grab does not'
+                         ' support hammer mode anymore.')
             del kwargs['hammer_timeouts']
 
         for key in kwargs:
@@ -397,7 +387,7 @@ class Grab(DeprecatedThings):
         self.request_counter = next(REQUEST_COUNTER)
         if kwargs:
             self.setup(**kwargs)
-        if not self.proxylist.is_empty() and self.config['proxy_auto_change']:
+        if self.proxylist.size() and self.config['proxy_auto_change']:
             self.change_proxy()
         self.request_method = self.detect_request_method()
         self.transport.process_config(self)
@@ -458,14 +448,14 @@ class Grab(DeprecatedThings):
         Process result of real request performed via transport extension.
         """
 
-        now = datetime.now()
+        now = datetime.utcnow()
         # TODO: move into separate method
         if self.config['debug_post']:
             post = self.config['post'] or self.config['multipart_post']
             if isinstance(post, dict):
                 post = list(post.items())
             if post:
-                if isinstance(post, basestring):
+                if isinstance(post, six.string_types):
                     post = post[:self.config['debug_post_limit']] + '...'
                 else:
                     items = normalize_http_values(post, charset='utf-8')
@@ -555,7 +545,7 @@ class Grab(DeprecatedThings):
             self.copy_request_data()
             self.save_dumps()
         except Exception as ex:
-            logging.error(unicode(ex))
+            logger.error(six.text_type(ex))
 
     def copy_request_data(self):
         # TODO: Maybe request object?
@@ -574,6 +564,10 @@ class Grab(DeprecatedThings):
 
         self.reset()
 
+        if isinstance(content, six.text_type):
+            raise error.GrabMisuseError('Method `setup_document` accepts only '
+                                        'byte string in `content` argument.')
+
         # Configure Document instance
         doc = Document(grab=self)
         doc.body = content
@@ -596,12 +590,13 @@ class Grab(DeprecatedThings):
         Set random proxy from proxylist.
         """
 
-        if not self.proxylist.is_empty():
+        if self.proxylist.size():
             proxy = self.proxylist.get_random_proxy()
-            self.setup(proxy=proxy.address, proxy_userpwd=proxy.userpwd,
+            self.setup(proxy=proxy.get_address(),
+                       proxy_userpwd=proxy.get_userpwd(),
                        proxy_type=proxy.proxy_type)
         else:
-            logging.debug('Proxy list is empty')
+            logger.debug('Proxy list is empty')
 
     """
     Private methods
@@ -736,20 +731,9 @@ class Grab(DeprecatedThings):
             headers = email.message_from_string('\n'.join(lines))
             return headers
         except Exception as ex:
-            logging.error('Could not parse request headers', exc_info=ex)
+            logger.error('Could not parse request headers', exc_info=ex)
             return {}
 
-    @property
-    def kit(self):
-        """
-        Return KitInterface object that provides some
-        methods to communicate with Kit transport.
-        """
-
-        if not self._kit:
-            self._kit = GrabKitInterface(self)
-        return self._kit
-
     def dump(self):
         """
         Shortcut for real-time debugging.
